@@ -98,7 +98,7 @@
  * </example>
  *
  * Since: 0.5.0
- **/
+ */
 
 #include <config.h>
 #include <glib.h>
@@ -209,7 +209,7 @@ gdata_download_stream_class_init (GDataDownloadStreamClass *klass)
 	 * The service which is used to authorize the download, and to which the download relates.
 	 *
 	 * Since: 0.5.0
-	 **/
+	 */
 	g_object_class_install_property (gobject_class, PROP_SERVICE,
 	                                 g_param_spec_object ("service",
 	                                                      "Service", "The service which is used to authorize the download.",
@@ -233,10 +233,10 @@ gdata_download_stream_class_init (GDataDownloadStreamClass *klass)
 	/**
 	 * GDataDownloadStream:download-uri:
 	 *
-	 * The URI of the file to download.
+	 * The URI of the file to download. This must be HTTPS.
 	 *
 	 * Since: 0.5.0
-	 **/
+	 */
 	g_object_class_install_property (gobject_class, PROP_DOWNLOAD_URI,
 	                                 g_param_spec_string ("download-uri",
 	                                                      "Download URI", "The URI of the file to download.",
@@ -254,7 +254,7 @@ gdata_download_stream_class_init (GDataDownloadStreamClass *klass)
 	 * or marshal the notification to the thread which owns the #GDataDownloadStream as appropriate.
 	 *
 	 * Since: 0.5.0
-	 **/
+	 */
 	g_object_class_install_property (gobject_class, PROP_CONTENT_TYPE,
 	                                 g_param_spec_string ("content-type",
 	                                                      "Content type", "The content type of the file being downloaded.",
@@ -272,7 +272,7 @@ gdata_download_stream_class_init (GDataDownloadStreamClass *klass)
 	 * or marshal the notification to the thread which owns the #GDataDownloadStream as appropriate.
 	 *
 	 * Since: 0.5.0
-	 **/
+	 */
 	g_object_class_install_property (gobject_class, PROP_CONTENT_LENGTH,
 	                                 g_param_spec_long ("content-length",
 	                                                    "Content length", "The length (in bytes) of the file being downloaded.",
@@ -293,7 +293,7 @@ gdata_download_stream_class_init (GDataDownloadStreamClass *klass)
 	 * call. The only way to cancel the download operation completely is using #GDataDownloadStream:cancellable.
 	 *
 	 * Since: 0.8.0
-	 **/
+	 */
 	g_object_class_install_property (gobject_class, PROP_CANCELLABLE,
 	                                 g_param_spec_object ("cancellable",
 	                                                      "Cancellable", "An optional cancellable used to cancel the entire download operation.",
@@ -355,9 +355,10 @@ gdata_download_stream_constructor (GType type, guint n_construct_params, GObject
 		priv->cancellable = g_cancellable_new ();
 	priv->network_cancellable_id = g_cancellable_connect (priv->cancellable, (GCallback) cancellable_cancel_cb, priv->network_cancellable, NULL);
 
-	/* Build the message */
+	/* Build the message. The URI must be HTTPS. */
 	_uri = soup_uri_new (priv->download_uri);
 	soup_uri_set_port (_uri, _gdata_service_get_https_port ());
+	g_assert_cmpstr (soup_uri_get_scheme (_uri), ==, SOUP_URI_SCHEME_HTTPS);
 	priv->message = soup_message_new_from_uri (SOUP_METHOD_GET, _uri);
 	soup_uri_free (_uri);
 
@@ -854,11 +855,24 @@ static gpointer
 download_thread (GDataDownloadStream *self)
 {
 	GDataDownloadStreamPrivate *priv = self->priv;
+	GDataAuthorizer *authorizer;
 
 	g_object_ref (self);
 
 	g_assert (priv->network_cancellable != NULL);
 
+	/* FIXME: Refresh authorization before sending message in order to prevent authorization errors during transfer.
+	 * See: https://gitlab.gnome.org/GNOME/libgdata/issues/23 */
+	authorizer = gdata_service_get_authorizer (priv->service);
+	if (authorizer) {
+		g_autoptr(GError) error = NULL;
+
+		gdata_authorizer_refresh_authorization (authorizer, priv->cancellable, &error);
+		if (error != NULL)
+			g_debug ("Error returned when refreshing authorization: %s", error->message);
+		else
+			gdata_authorizer_process_request (authorizer, priv->authorization_domain, priv->message);
+	}
 	/* Connect to the got-headers signal so we can notify clients of the values of content-type and content-length */
 	g_signal_connect (priv->message, "got-headers", (GCallback) got_headers_cb, self);
 	g_signal_connect (priv->message, "got-chunk", (GCallback) got_chunk_cb, self);
@@ -928,7 +942,7 @@ reset_network_thread (GDataDownloadStream *self)
  * gdata_download_stream_new:
  * @service: a #GDataService
  * @domain: (allow-none): the #GDataAuthorizationDomain to authorize the download, or %NULL
- * @download_uri: the URI to download
+ * @download_uri: the URI to download; this must be HTTPS
  * @cancellable: (allow-none): a #GCancellable for the entire download stream, or %NULL
  *
  * Creates a new #GDataDownloadStream, allowing a file to be downloaded from a GData service using standard #GInputStream API.
@@ -945,7 +959,7 @@ reset_network_thread (GDataDownloadStream *self)
  * Return value: a new #GInputStream, or %NULL; unref with g_object_unref()
  *
  * Since: 0.9.0
- **/
+ */
 GInputStream *
 gdata_download_stream_new (GDataService *service, GDataAuthorizationDomain *domain, const gchar *download_uri, GCancellable *cancellable)
 {
@@ -971,7 +985,7 @@ gdata_download_stream_new (GDataService *service, GDataAuthorizationDomain *doma
  * Return value: (transfer none): the #GDataService used to authorize the download
  *
  * Since: 0.5.0
- **/
+ */
 GDataService *
 gdata_download_stream_get_service (GDataDownloadStream *self)
 {
@@ -1006,7 +1020,7 @@ gdata_download_stream_get_authorization_domain (GDataDownloadStream *self)
  * Return value: the URI of the file being downloaded
  *
  * Since: 0.5.0
- **/
+ */
 const gchar *
 gdata_download_stream_get_download_uri (GDataDownloadStream *self)
 {
@@ -1024,7 +1038,7 @@ gdata_download_stream_get_download_uri (GDataDownloadStream *self)
  * Return value: the content type of the file being downloaded, or %NULL
  *
  * Since: 0.5.0
- **/
+ */
 const gchar *
 gdata_download_stream_get_content_type (GDataDownloadStream *self)
 {
@@ -1050,7 +1064,7 @@ gdata_download_stream_get_content_type (GDataDownloadStream *self)
  * Return value: the content length of the file being downloaded, or <code class="literal">-1</code>
  *
  * Since: 0.5.0
- **/
+ */
 gssize
 gdata_download_stream_get_content_length (GDataDownloadStream *self)
 {
@@ -1076,7 +1090,7 @@ gdata_download_stream_get_content_length (GDataDownloadStream *self)
  * Return value: (transfer none): the #GCancellable for the entire download operation
  *
  * Since: 0.8.0
- **/
+ */
 GCancellable *
 gdata_download_stream_get_cancellable (GDataDownloadStream *self)
 {
